#!/usr/local/bin/perl -w

#
# Convert HAC4 tur-files to Polar hrm-files
#
# Filename: tur2hrm.pl
# Author  : Michael Skerwiderski
# Version : 1.0.0 (13.07.2004)
#

#
# Usage: tur2hrm.pl <inputfilename> <outputfilename>
#

use strict;

my $CRLF = "\n";

my $rec_interval_secs = 20;

sub run($$)
{  
  print("converting HAC4 tur-file to Polar hrm-file ... \n");

  my $src = $_[0];
  my $dest = $_[1];
  
  my $fin = open(INPUT, $src);
  my $fout = open(OUTPUT, "> ".$dest);

  if (not defined($fin) or not defined($fout))
  { die "Error: can't open file '".$fin."-".$fout."'!\n"; }
    
  convert();

  close(OUTPUT);
  close(INPUT);

  print("hrm-file successfully generated!\n");  
}

sub convert_date($) # dd.mm.yyyy --> yyyymmdd
{
  my $src = $_[0];
  return substr($src, -4, 4).substr($src, 3, 2).substr($src, 0, 2);
}

sub convert_time($) # HH:MM --> HH:MM:SS.M
{
  return $_[0].":00.0";
}

sub convert_duration($) # secs --> HH:MM:SS.M
{
  my $cnt_sec = $_[0];
  my $hrs = int($cnt_sec / 3600);
  my $min = int(($cnt_sec - ($hrs * 3600))/ 60);
  my $sec = int (($cnt_sec - ($hrs * 3600) - ($min * 60)));
  return sprintf("%02d", $hrs).":".sprintf("%02d", $min).":".sprintf("%02d", $sec).".0";
}

sub convert()
{
  my $i = "";
  my $line = 0;

  my $datum = "";
  my $starttime = "";
  my $duration = "";
  my $cntrecs = 0;
    
  my $in_data_section = 0;

  while (!$in_data_section && defined($i = <INPUT>))
  {
    $line = $line + 1;
    
    chomp $i;

    if ($line == 8)
    {
      $datum = convert_date($i);
    }
    elsif ($line == 9)
    {
      $starttime = convert_time($i);
    }
    elsif ($line == 12)
    {
      $duration = convert_duration($i);
    }
    elsif ($line == 14)
    {
      print "altitude (max): ".$i."\n";
    }
    elsif ($line == 15)
    {
      print "altitude (diff): ".$i."\n";
    }
    elsif ($line == 18)
    {
      print "speed (avg): ".$i."\n";
    }
    elsif ($line == 19)
    {
      print "temperature (avg): ".$i."\n";
    }
    elsif ($line == 20)
    {
      print "heartrate (avg): ".$i."\n";
    }
    elsif ($line == 54)
    {
      write_params_section($datum, $starttime, $duration, 5);

      $cntrecs = $i;
      print "count of records: ".$cntrecs."\n";
      $in_data_section = 1;

    }
  }

  if ($in_data_section)
  {
    write_trip_section();
    write_header_section("HRData");
    
    my $isfirst = 1;
    my $prev_distance = 0;
    my $prev_speed = 0;
    my $prev_altitude = 0;
    my $prev_heartrate = 0; 

    while ($cntrecs > 0)
    { 
      get_bytes(8);
      my $distance = get_distance();
      my $speed = get_speed($prev_distance, $distance);
      $prev_distance = $distance;
      my $altitude = get_altitude();
      my $heartrate = get_heartrate();  
      get_bytes(5);    
      
      if (!$isfirst)
      {
        write_app_hrdata_lines($prev_heartrate, $prev_speed, $prev_altitude, $heartrate, $speed, $altitude);
      }

      write_hrdata_line($heartrate, $speed, $altitude);

      $prev_speed = $speed;
      $prev_altitude = $altitude;
      $prev_heartrate = $heartrate;

      $isfirst = 0;
      $cntrecs = $cntrecs - 1;      
    }    
  }
}

sub write_app_hrdata_lines($$$$$$)
{
  my $prev_heartrate = $_[0];
  my $prev_speed = $_[1];
  my $prev_altitude = $_[2];
  my $heartrate = $_[3];
  my $speed = $_[4];
  my $altitude = $_[5];

  my @vals_heartrate = get_app_vals($prev_heartrate, $heartrate, 3);
  my @vals_speed = get_app_vals($prev_speed, $speed, 3);
  my @vals_altitude = get_app_vals($prev_altitude, $altitude, 3);

  write_hrdata_line($vals_heartrate[0], $vals_speed[0], $vals_altitude[0]);
  write_hrdata_line($vals_heartrate[1], $vals_speed[1], $vals_altitude[1]);
  write_hrdata_line($vals_heartrate[2], $vals_speed[2], $vals_altitude[2]);  
}

sub get_app_vals($$$)
{
  my $prev_val = $_[0];
  my $val = $_[1];
  my $cnt_vals = $_[2];

  my $step = ($val - $prev_val) / ($cnt_vals + 1);

  my @vals = ();

  for (my $i=0; $i < $cnt_vals; $i++)
  {
    push(@vals, sprintf("%.0f", $prev_val + ($i+1) * $step));  
  }

  return @vals;
}

sub get_distance() # in meter
{
  my $bytes = get_bytes(4);
  return hex($bytes) * 10;
}

sub get_speed($$) # in dezimeter/hr
{
  my $distance = (3600 / $rec_interval_secs) * ($_[1] - $_[0]);  
  my $speed = sprintf ("%.0f", $distance / 100);  
  return $speed;
}

sub get_altitude() # meter
{
  my $bytes = get_bytes(2);
  return hex($bytes);
}

sub get_heartrate() # beats per minute
{
  my $bytes = get_bytes(1);
  return hex($bytes);
}

sub get_bytes($)
{
  my $cnt = $_[0];
  my $buf = "";
  my $dummy = "";
  my @values = ();

  my $template = "";

  for (my $i=0; $i < $cnt; $i++)
  {
    $template = $template." H2";
  }

  read(INPUT, $buf, $cnt);  
  (@values) = unpack($template,$buf);

  my $bytes = "";
  for (my $i=0; $i < $cnt; $i++)
  {
    $bytes = $bytes.$values[$cnt-$i-1];
  }
  
  return $bytes;
}

sub dump_bytes($@)
{
  my @values = @_;

  print $values[0].": ";

  for (my $i=1; $i < @values; $i++)
  {    
    print "[".$values[$i]."]";   
  }
  
  print "\n";
}

sub write_header_section($)
{
  my $header = $_[0];
  print OUTPUT "[".$header."]".$CRLF;  
}

sub write_params_section($$$$)
{
  my $date      = $_[0]; # yyyymmdd
  my $starttime = $_[1]; # hh:mm:ss.m
  my $duration  = $_[2]; # hh:mm:ss.m
  my $interval  = $_[3]; # e.g. 5

  write_header_section("Params");

  print OUTPUT "Version=106".$CRLF;
  print OUTPUT "Monitor=99".$CRLF;
  print OUTPUT "SMode=101000100".$CRLF;
  print OUTPUT "Date=".$date.$CRLF;
  print OUTPUT "StartTime=".$starttime.$CRLF;
  print OUTPUT "Length=".$duration.$CRLF;
  print OUTPUT "Interval=".$interval.$CRLF;
  print OUTPUT "ActiveLimit=0".$CRLF;
  print OUTPUT "MaxHR=0".$CRLF;
  print OUTPUT "RestHR=0".$CRLF;
  print OUTPUT "StartDelay=0".$CRLF.$CRLF;
}

sub write_trip_section()
{
  write_header_section("Trip"); 
  
  print OUTPUT "0".$CRLF;
  print OUTPUT "0".$CRLF;
  print OUTPUT "0".$CRLF;
  print OUTPUT "0".$CRLF;
  print OUTPUT "0".$CRLF;
  print OUTPUT "0".$CRLF;
  print OUTPUT "0".$CRLF;
  print OUTPUT "0".$CRLF.$CRLF;  
}
                 
sub write_hrdata_line($$$) # heartrate, speed, altitude
{
  my $hr  = $_[0];
  my $spd = $_[1];
  my $alt = $_[2]; 

  print OUTPUT $hr."\t".$spd."\t".$alt.$CRLF;
}

run($ARGV[0], $ARGV[1]);
